"""Module for the workflow models."""

from pydantic import BaseModel, Field

from janus.constants.FileTag import FileTag
from janus.dto.collect_qc_request import WorkflowInfo
from janus.models.multiqc.models import (
    Somalier,
    PicardAlignmentSummary,
    PicardDuplicates,
    PicardHsMetrics,
    PicardInsertSize,
    PicardWGSMetrics,
    SamtoolsStats,
    Fastp,
)


class BalsamicSample(BaseModel):
    sample_id: str
    alignment_summary_metrics: PicardAlignmentSummary | None = Field(
        ..., alias=FileTag.ALIGNMENT_SUMMARY_METRICS
    )
    duplicates: PicardDuplicates = Field(..., alias=FileTag.DUPLICATES)
    wgs_metrics: PicardWGSMetrics | None = Field(None, alias=FileTag.WGS_METRICS)
    hs_metrics: PicardHsMetrics = Field(..., alias=FileTag.HS_METRICS)
    insert_size: PicardInsertSize = Field(..., alias=FileTag.INSERT_SIZE)
    samtools_stats: SamtoolsStats = Field(..., alias=FileTag.SAMTOOLS_STATS)
    fastp: Fastp = Field(..., alias=FileTag.FASTP)


class Balsamic(BaseModel):
    samples: list[BalsamicSample]
    somalier: Somalier | None = None
    workflow: WorkflowInfo



from pydantic import BaseModel

from janus.models.workflow.balsamic import Balsamic


class CollectQCResponse(BaseModel):
    """Collect QC response model."""

    case_id: str
    case_info: Balsamic



"""Constant for the workflows."""

from enum import StrEnum


class Workflow(StrEnum):
    BALSAMIC: str = "balsamic"
    BALSAMIC_QC: str = "balsamic-qc"
    BALSAMIC_UMI: str = "balsamic-umi"

    @classmethod
    def values(cls) -> list[str]:
        return [member.value for member in cls]



"""Read JSON module"""

import json
from pathlib import Path


class OrderedDictWithUnchangedOrder(dict):
    def __setitem__(self, key, value):
        if key not in self:
            super(OrderedDictWithUnchangedOrder, self).__setitem__(key, value)


def read_json(file_path: Path) -> any:
    """Read content in a json file."""
    with open(file_path, "r") as file:
        return json.load(file, object_pairs_hook=OrderedDictWithUnchangedOrder)



class JanusError(Exception):
    def __init__(self, message: str = ""):
        super(JanusError, self).__init__()
        self.message = message


class ParseJSONError(JanusError):
    """Raises an error in case of an error when parsing a JSON file."""


class WorkflowNotSupportedError(JanusError):
    """Raises an error in case a request for an unsupported workflow was made."""
